class Deck::CommandLine::ContainerHelper
  attr_reader :container

  def initialize(name, options)
    @options = options

    load_blueprint name
  end

  def run_command
    command = @container.run_command
    process_meta command
  end

  def build_command
    command = raw_build_command
    process_meta command
  end

  def update_latest_tag
    process_meta "docker tag -f #{@container.image_name} #{@container.repository}/#{@container.name}:latest"
  end

  private
  def load_blueprint(name)
    @container = Deck.send(:eval, File.read("#{@options[:dir]}/#{name}.deck"))

    process_variables
  end

  def process_meta(command)
    result = command
    @container.meta.each do |_, v|
      result = v.process(result) if v.respond_to? :process
    end

    result
  end

  def process_variables
    Hash[*(@options[:value].split(',').map { |item| item.split('=') }.flatten)].each do |key, value|
      @container.meta[:variable].set key, value
    end unless @options[:value].nil? or @options[:value].empty?

    @container
  end

  def raw_build_command
    <<-EOF
docker build --rm -t #{@container.image_name} - <<DOCKERFILE
#{@container.build_command}
DOCKERFILE
    EOF
  end
end
