class Deck::Container
  autoload :Repository, 'deck/container/repository'
  autoload :Name,       'deck/container/name'
  autoload :From,       'deck/container/from'
  autoload :Expose,     'deck/container/expose'
  autoload :Cmd,        'deck/container/cmd'
  autoload :Archive,    'deck/container/archive'
  autoload :Add,        'deck/container/add'
  autoload :Start,      'deck/container/start'
  autoload :WorkDir,    'deck/container/work_dir'
  autoload :Tag,        'deck/container/tag'
  autoload :Variable,   'deck/container/variable'
  autoload :Publish,    'deck/container/publish'
  autoload :Env,        'deck/container/env'

  attr_reader :meta

  def initialize
    @information = {}
    @meta = {}
    @call_order = []
    self.class.constants.each do |sub|
      sub = self.class.const_get(sub).new
      if sub.class.meta?
        @meta[sub.class.register_name] = sub
      else
        @information[sub.class.register_name] = sub
      end
    end
  end

  def build_command
    dockerfile = ''
    @call_order.each do |info|
      command = @information[info].build_command(@information)
      dockerfile += "#{command}\n" unless command.nil?
      @information[info].next if @information[info].class.include? Deck::Repeatable
    end
    dockerfile
  end

  def image_name
    "#{repository.to_s}/#{name.to_s}:#{tag.to_s}"
  end

  def run_command
    "docker run #{image_name}"
  end

  def method_missing(name, *args, &block)
    return handle_information(name, *args, &block) if @information.include? name.to_sym
    return handle_meta(name, *args, &block) if @meta.include? name.to_sym
    raise NoMethodError, "do not define such method : #{name.inspect}"
  end

  private
  def handle_information(name, *args, &block)
    if args.nil? || args.empty?
      @information[name.to_sym].send :get
    else
      info = @information[name.to_sym]
      value = info.send :set, *args
      @call_order << name.to_sym unless @call_order.include? name.to_sym and not info.class.repeatable?
      value
    end
  end

  def handle_meta(name, *args, &block)
    if args.nil? || args.empty?
      @meta[name.to_sym].send :get
    else
      @meta[name.to_sym].send :set, *args
    end
  end
end
